# KotonohaLink Plug-in
# Copyright 2005 CHEEBOW (Motokazu Sekine) All Right Reserved.
# $Id: KotonohaLink.pl 33 2005-10-30 03:18:20Z cheebow $

package MT::plugins::KotonohaLink;

use strict;

use MT::Template::Context;
use MT::ConfigMgr;
use MT::I18N;
use MT::Plugin;
@MT::Plugin::KotonohaLink::ISA = qw(MT::Plugin);

my $PLUGIN_NAME = 'KotonohaLink';
my $VERSION = '0.61';

my $DEFAULT_LINKS =<< 'LINKS';
Movable ?Type||http://narodlink.ru/
Реферат?||http://claw.ru/
Рыба?||http://doyounow.ru/
База||http://doyounow.ru/
Форум||http://XN--80AAC1EFK.XN--P1AI/
LINKS

###----------------------------------------------------------------------###
my $plugin = new MT::Plugin::KotonohaLink( {
    name => $PLUGIN_NAME,
    version => $VERSION,
    description => 'Plug-in which changes a character sequence with a link.',
    doc_link => 'http://labs.m-logic.jp/plugins/kotonohalink/docs/kotonohalink.html',
    author_name => 'M-Logic, Inc.',
    author_link => 'http://labs.m-logic.jp/',
    config_template => \&template,
    settings => new MT::PluginSettings([
        ['kotonoha_link_words', { Default => $DEFAULT_LINKS }]
        ]),
    });
MT->add_plugin($plugin);


###----------------------------------------------------------------------###
MT::Template::Context->add_global_filter(KotonohaLink => \&KotonohaLink );

###----------------------------------------------------------------------###
sub instance { $plugin; }

sub template {
    my $tmpl = <<'EOT';
    <div class="setting">
    <div class="label">
    <label for="kotonoha_link_words"><MT_TRANS phrase="word and link:"></label>
    </div>
    <div class="field">
    <p><textarea name="kotonoha_link_words" id="kotonoha_link_words" cols="60" rows="10"><TMPL_VAR NAME=KOTONOHA_LINK_WORDS ESCAPE=HTML></textarea></p>
    </div> 
    </div>
EOT
}

sub kotonoha_link_words {
    my ($blog_id) = @_;
    my %plugin_param;

    $plugin->load_config(\%plugin_param, 'blog:'.$blog_id);
    my $key = $plugin_param{kotonoha_link_words};
    unless ($key) {
        $plugin->load_config(\%plugin_param, 'system');
        $key = $plugin_param{kotonoha_link_words};
    }
    $key;
}

###----------------------------------------------------------------------###
sub KotonohaLink {
    my ($text, $arg, $ctx) = @_;
    $arg or return $text;
    
    my $cfg = MT::ConfigMgr->instance;
    my $blog_id = $ctx->stash('blog')->id;

    # Load Data
    our %wordList;
    my $data = kotonoha_link_words($blog_id);
    my @lines = split(/\n/, $data);
    foreach my $line (@lines) {
        $line =~ s/[\r\n]//g;
        last if !$line;
        my($word, $url) = split(/\|\|/, $line);
        $wordList{$word} = $url;
    }

    # Filter
    use HTML::Parser;
    
    my $p = HTML::Parser->new(
        api_version => 3,
        default_h => [\&default, "text"],
        start_h   => [\&start,   "tagname, text"],
        end_h     => [\&end,     "tagname, text"],
        text_h    => [\&text,    "text"],
        marked_sections => 1,
    );
    $p->unbroken_text(1);

    our $out_text = "";
    our $tmp_text = $text;

    foreach our $cur_word(keys(%wordList)) {
        our $add_link = 1;
        $out_text = "";
        $p->parse($tmp_text)->eof;
        $tmp_text = $out_text;
    }
    
    $out_text;
}

###----------------------------------------------------------------------###
sub default(){
    our $out_text .= shift;
}

sub start(){
    my ($tagname, $str) = @_;
    our ($add_link, $out_text);
    if($tagname eq "a"){
        $add_link = 0;
    }
    $out_text .= $str;
}

sub end(){
    my ($tagname, $str) = @_;
    our ($add_link, $out_text);
    if($tagname eq "a"){
        $add_link = 1;
    }
    $out_text .= $str;
}

sub text(){
    my $str = shift;
    our ($add_link, $out_text);
    &add_link(\$str) if $add_link;
    $out_text .= $str;
}

sub add_link(){
    my $str_ref = shift;
    our %wordList;
    our $cur_word;
    $$str_ref =~ s!($cur_word)!<a class="KotonohaLink" href="$wordList{$cur_word}">$1</a>!g;
}
